
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'FlutterBaseNativeAd.dart';
import 'FlutterBidmadCommon.dart';

class BidmadNativeAdWidget extends StatefulWidget {
  final BidmadNativeAdWidgetCreatedCallback onBidmadNativeAdWidgetCreated;
  final String layoutName;

  const BidmadNativeAdWidget({
    Key? key,
    required this.layoutName,
    required this.onBidmadNativeAdWidgetCreated,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => BidmadNativeAdWidgetState(this.layoutName);
}

typedef BidmadNativeAdWidgetCreatedCallback = void Function(FlutterBaseNativeAd controller);

class BidmadNativeAdWidgetState extends State<BidmadNativeAdWidget> {
  FlutterBidmadCommon common = FlutterBidmadCommon();
  String defaultChannelNm = "com.bidmad.sdk.nativead/Widget_";
  String layoutName;
  late FlutterBaseNativeAd nativeAd;


  BidmadNativeAdWidgetState(this.layoutName);

  @override
  void dispose() {
    print("BidmadNativeAdWidget dispose");
    if(nativeAd != null){
      nativeAd.removeWidget();
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    // This is used in the platform side to register the view.
    common.initNativeAdWidget(this.layoutName);

    final String viewType = 'BidmadNativeAdWidget';
    // Pass parameters to the platform side.
    final Map<String, dynamic> creationParams = <String, dynamic>{};

    if(defaultTargetPlatform == TargetPlatform.android) {
      return AndroidView(
        viewType: viewType,
        creationParams: creationParams,
        onPlatformViewCreated: _onPlatformViewCreated,
        creationParamsCodec: const StandardMessageCodec(),
      );
    } else if(defaultTargetPlatform == TargetPlatform.iOS) {
      return UiKitView(
        viewType: viewType,
        creationParams: creationParams,
        onPlatformViewCreated: _onPlatformViewCreated,
        creationParamsCodec: const StandardMessageCodec(),
      );
    } else{
      throw Exception("Not Support Platform");
    }
  }
  void _onPlatformViewCreated(int id) {

    print("_onPlatformViewCreated : " + id.toString());
    if (widget.onBidmadNativeAdWidgetCreated == null) {
      return;
    }
    nativeAd = FlutterBaseNativeAd(channelName: defaultChannelNm+id.toString());
    widget.onBidmadNativeAdWidgetCreated(nativeAd);
  }
}