import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

class FlutterBaseReward{
  late String _channelNm;
  late MethodChannel _platform;

  var onLoadAdCallback;
  var onShowAdCallback;
  var onFailAdCallback;
  var onCompleteAdCallback;
  var onCloseAdCallback;
  var onClickAdCallback;
  var onSkipAdCallback;

  FlutterBaseReward({
    required String channelName
  }){
    this._channelNm = channelName;
    _initChannel(_channelNm);
  }

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
    _platform.setMethodCallHandler(_handleMethod);
  }

  Future<void> load() async {
    await _platform.invokeMethod('load');
  }

  Future<void> show() async {
    await _platform.invokeMethod('show');
  }

  Future<bool> isLoaded() async {
    return await _platform.invokeMethod('isLoaded');
  }

  Future<void> setAdInfo(String zoneId) async {
    await _platform.invokeMethod('makeReward', {"zoneId":zoneId});
  }

  Future<void> setCallbackListener({
    void Function()? onLoadAd,
    void Function()? onShowAd,
    void Function(String errorMsg)? onFailAd,
    void Function()? onCompleteAd,
    void Function()? onCloseAd,
    void Function()? onClickAd,
    void Function()? onSkipAd
  }) async {

    if(onLoadAd != null){
      onLoadAdCallback = onLoadAd;
    }
    if(onShowAd != null){
      onShowAdCallback = onShowAd;
    }
    if(onFailAd != null){
      onFailAdCallback = onFailAd;
    }
    if(onCompleteAd != null){
      onCompleteAdCallback = onCompleteAd;
    }
    if(onCloseAd != null){
      onCloseAdCallback = onCloseAd;
    }
    if(onClickAd != null){
      onClickAdCallback = onClickAd;
    }
    if(onSkipAd != null){
      onSkipAdCallback = onSkipAd;
    }
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    switch (call.method) {
      case 'onLoadAd':
        if (onLoadAdCallback != null) {
          onLoadAdCallback();
        };
        break;
      case 'onShowAd':
        if (onShowAdCallback != null) {
          onShowAdCallback();
        };
        break;
      case 'onFailAd':
        String errorMessage = call.arguments.toString();
        if (onFailAdCallback != null) {
          onFailAdCallback(errorMessage);
        };
        break;
      case 'onCompleteAd':
        if (onCompleteAdCallback != null) {
          onCompleteAdCallback();
        };
        break;
      case 'onCloseAd':
        if (onCloseAdCallback != null) {
          onCloseAdCallback();
        };
        break;
      case 'onClickAd':
        if (onClickAdCallback != null) {
          onClickAdCallback();
        };
        break;
      case 'onSkipAd':
        if (onSkipAdCallback != null) {
          onSkipAdCallback();
        };
        break;
      default:
        throw ('method not defined');
    }
  }
}